@@ -11,7 +11,7 @@
     Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
     Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski
 
-    $FileInfo: __init__.py - Last Update: 2/19/2023 Ver. 2.7.22 RC 1 - Author: cooldude2k $
+    $FileInfo: __init__.py - Last Update: 3/8/2023 Ver. 2.7.24 RC 1 - Author: cooldude2k $
 '''
 
 from __future__ import absolute_import, division, print_function, unicode_literals;
@@ -45,4 +45,4 @@ if(pilsupport):
 else:
  cairosupport = upcean.support.check_for_cairo();
 if(pilsupport or cairosupport):
- import upcean.fonts, upcean.xml, upcean.barcodes, upcean.getsfname;
+ import upcean.fonts, upcean.xml, upcean.barcodes, upcean.barcodes.getsfname;
