@@ -11,68 +11,80 @@
     Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
     Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski
 
-    $FileInfo: code93.py - Last Update: 2/19/2023 Ver. 2.7.22 RC 1 - Author: cooldude2k $
+    $FileInfo: code93.py - Last Update: 3/8/2023 Ver. 2.7.24 RC 1 - Author: cooldude2k $
 '''
 
 from __future__ import absolute_import, division, print_function, unicode_literals;
-import re, os, sys, types, upcean.getsfname, upcean.support;
+import re, os, sys, types, upcean.barcodes.getsfname, upcean.support;
 pilsupport = upcean.support.check_for_pil();
+cairosupport = upcean.support.check_for_cairo();
+from upcean.barcodes.predraw import *;
 if(pilsupport):
- cairosupport = False;
-else:
- cairosupport = upcean.support.check_for_cairo();
-if(pilsupport):
- from upcean.barcodes.prepil import *;
- from PIL import Image, ImageDraw, ImageFont;
+ import upcean.barcodes.prepil;
 if(cairosupport):
- from upcean.precairo import *;
- import cairo;
+ import upcean.barcodes.precairo;
 
-def create_code93_barcode(upc,outfile="./code93.png",resize=1,hideinfo=(False, False, False),barheight=(48, 54),textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255))):
+def create_code93_barcode(upc,outfile="./code93.png",resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=1,textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
  upc = str(upc);
  hidesn = hideinfo[0];
  hidecd = hideinfo[1];
  hidetext = hideinfo[2];
+ imageoutlib = imageoutlib.lower();
+ if(not pilsupport and imageoutlib=="pillow"):
+  imageoutlib = "cairo";
+ if(not cairosupport and imageoutlib=="cairo"):
+  imageoutlib = "pillow";
+ if(imageoutlib!="pillow" and imageoutlib!="cairo"):
+  imageoutlib = "pillow";
+ if(not pilsupport and not cairosupport):
+  return False;
  if(len(upc) < 1): 
   return False;
+ if(barwidth < 1): 
+  barwidth = 1;
  if(not re.findall("([0-9a-zA-Z\-\.\$\/\+% ]+)", upc)):
   return False;
  if(not re.findall("^([0-9]*[\.]?[0-9])", str(resize)) or int(resize) < 1):
   resize = 1;
- try:
-  pil_ver = Image.PILLOW_VERSION;
-  pil_ver = pil_ver.split(".");
-  pil_ver = [int(x) for x in pil_ver];
-  pil_is_pillow = True;
- except AttributeError:
+ if(pilsupport and imageoutlib=="pillow"):
   try:
-   pil_ver = Image.VERSION;
-   pil_is_pillow = False;
+   pil_ver = Image.PILLOW_VERSION;
+   pil_ver = pil_ver.split(".");
+   pil_ver = [int(x) for x in pil_ver];
+   pil_is_pillow = True;
   except AttributeError:
-   pil_ver = Image.__version__;
-   pil_is_pillow = True;
+   try:
+    pil_ver = Image.VERSION;
+    pil_is_pillow = False;
+   except AttributeError:
+    pil_ver = Image.__version__;
+    pil_is_pillow = True;
+   except NameError:
+    pil_ver = Image.__version__;
+    pil_is_pillow = True;
+   pil_ver = pil_ver.split(".");
+   pil_ver = [int(x) for x in pil_ver];
   except NameError:
-   pil_ver = Image.__version__;
-   pil_is_pillow = True;
-  pil_ver = pil_ver.split(".");
-  pil_ver = [int(x) for x in pil_ver];
- except NameError:
-  try:
-   pil_ver = Image.VERSION;
-   pil_is_pillow = False;
-  except AttributeError:
-   pil_ver = Image.__version__;
-   pil_is_pillow = True;
-  except NameError:
-   pil_ver = Image.__version__;
-   pil_is_pillow = True;
-  pil_ver = pil_ver.split(".");
-  pil_ver = [int(x) for x in pil_ver];
- pil_addon_fix = 0;
- pil_prevercheck = [str(x) for x in pil_ver];
- pil_vercheck = int(pil_prevercheck[0]+pil_prevercheck[1]+pil_prevercheck[2]);
- if(pil_is_pillow and pil_vercheck>=210 and pil_vercheck<220):
-  pil_addon_fix = int(resize) * 2;
+   try:
+    pil_ver = Image.VERSION;
+    pil_is_pillow = False;
+   except AttributeError:
+    pil_ver = Image.__version__;
+    pil_is_pillow = True;
+   except NameError:
+    pil_ver = Image.__version__;
+    pil_is_pillow = True;
+   pil_ver = pil_ver.split(".");
+   pil_ver = [int(x) for x in pil_ver];
+  pil_addon_fix = 0;
+  pil_prevercheck = [str(x) for x in pil_ver];
+  pil_vercheck = int(pil_prevercheck[0]+pil_prevercheck[1]+pil_prevercheck[2]);
+  if(pil_is_pillow and pil_vercheck>=210 and pil_vercheck<220):
+   pil_addon_fix = int(resize) * 2;
+ elif(pilsupport and imageoutlib=="pillow"):
+  pil_addon_fix = 0;
+ else:
+  pil_addon_fix = 0;
  upc = upc.upper();
  upc_matches = list(upc);
  if(len(upc_matches)<=0):
@@ -104,42 +116,34 @@ def create_code93_barcode(upc,outfile="./code93.png",resize=1,hideinfo=(False, F
   UPC_Count += 1; 
   UPC_Weight += 1;
  upc_matches.append(Code93Array[UPC_Sum % 47]);
- upc_size_add = (len(upc_matches) * 9);
- if(pilsupport):
-  upc_preimg = Image.new("RGB", (37 + upc_size_add, barheight[1] + 9));
+ upc_size_add = (len(upc_matches) * 9) * barwidth;
+ if(pilsupport and imageoutlib=="pillow"):
+  upc_preimg = Image.new("RGB", ((37 * barwidth) + upc_size_add, barheight[1] + 9));
   upc_img = ImageDraw.Draw(upc_preimg);
-  upc_img.rectangle([(0, 0), (37 + upc_size_add, barheight[1] + 9)], fill=barcolor[2]);
- if(cairosupport):
-  upc_preimg = cairo.ImageSurface(cairo.FORMAT_RGB24, 37 + upc_size_add, barheight[1] + 8);
+  upc_img.rectangle([(0, 0), ((37 * barwidth) + upc_size_add, barheight[1] + 9)], fill=barcolor[2]);
+ if(cairosupport and imageoutlib=="cairo"):
+  upc_preimg = cairo.ImageSurface(cairo.FORMAT_RGB24, (37 * barwidth) + upc_size_add, barheight[1] + 8);
   upc_img = cairo.Context (upc_preimg);
   upc_img.set_antialias(cairo.ANTIALIAS_NONE);
-  upc_img.rectangle(0, 0, 37 + upc_size_add, barheight[1] + 8);
+  upc_img.rectangle(0, 0, (37 * barwidth) + upc_size_add, barheight[1] + 8);
   upc_img.set_source_rgb(barcolor[2][0], barcolor[2][1], barcolor[2][2]);
   upc_img.fill();
  upc_array = { 'upc': upc, 'code': [ ] };
  LineSize = barheight[0];
  if(hidetext):
   LineSize = barheight[1];
- drawColorLine(upc_img, 0, 4, 0, LineSize, barcolor[2]);
- drawColorLine(upc_img, 1, 4, 1, LineSize, barcolor[2]);
- drawColorLine(upc_img, 2, 4, 2, LineSize, barcolor[2]);
- drawColorLine(upc_img, 3, 4, 3, LineSize, barcolor[2]);
- drawColorLine(upc_img, 4, 4, 4, LineSize, barcolor[2]);
- drawColorLine(upc_img, 5, 4, 5, LineSize, barcolor[2]);
- drawColorLine(upc_img, 6, 4, 6, LineSize, barcolor[2]);
- drawColorLine(upc_img, 7, 4, 7, LineSize, barcolor[2]);
- drawColorLine(upc_img, 8, 4, 8, LineSize, barcolor[2]);
- drawColorLine(upc_img, 9, 4, 9, LineSize, barcolor[0]);
- drawColorLine(upc_img, 10, 4, 10, LineSize, barcolor[2]);
- drawColorLine(upc_img, 11, 4, 11, LineSize, barcolor[0]);
- drawColorLine(upc_img, 12, 4, 12, LineSize, barcolor[2]);
- drawColorLine(upc_img, 13, 4, 13, LineSize, barcolor[0]);
- drawColorLine(upc_img, 14, 4, 14, LineSize, barcolor[0]);
- drawColorLine(upc_img, 15, 4, 15, LineSize, barcolor[0]);
- drawColorLine(upc_img, 16, 4, 16, LineSize, barcolor[0]);
- drawColorLine(upc_img, 17, 4, 17, LineSize, barcolor[2]);
+ start_barcolor = [0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0];
+ LineStart = 0;
+ BarNum = 0;
+ start_bc_num_end = len(start_barcolor);
+ while(BarNum < start_bc_num_end):
+  if(start_barcolor[BarNum]==1):
+   drawColorLine(upc_img, LineStart, 4, LineStart, LineSize, barwidth, barcolor[0], imageoutlib);
+  if(start_barcolor[BarNum]==0):
+   drawColorLine(upc_img, LineStart, 4, LineStart, LineSize, barwidth, barcolor[2], imageoutlib);
+  LineStart += barwidth;
+  BarNum += 1;
  NumZero = 0; 
- LineStart = 18; 
  while (NumZero < len(upc_matches)):
   left_barcolor = [1, 0, 0, 0, 1, 0, 1, 0, 0];
   if(upc_matches[NumZero]=="0"):
@@ -255,44 +259,48 @@ def create_code93_barcode(upc,outfile="./code93.png",resize=1,hideinfo=(False, F
   InnerUPCNum = 0;
   while (InnerUPCNum < len(left_barcolor)):
    if(left_barcolor[InnerUPCNum]==1):
-    drawColorLine(upc_img, LineStart, 4, LineStart, LineSize, barcolor[0]);
+    drawColorLine(upc_img, LineStart, 4, LineStart, LineSize, barwidth, barcolor[0], imageoutlib);
    if(left_barcolor[InnerUPCNum]==0):
-    drawColorLine(upc_img, LineStart, 4, LineStart, LineSize, barcolor[2]);
-   LineStart += 1;
+    drawColorLine(upc_img, LineStart, 4, LineStart, LineSize, barwidth, barcolor[2], imageoutlib);
+   LineStart += barwidth;
+   BarNum += 1;
    InnerUPCNum += 1;
   NumZero += 1; 
- drawColorLine(upc_img, 18 + upc_size_add, 4, 18 + upc_size_add, LineSize, barcolor[0]);
- drawColorLine(upc_img, 19 + upc_size_add, 4, 19 + upc_size_add, LineSize, barcolor[2]);
- drawColorLine(upc_img, 20 + upc_size_add, 4, 20 + upc_size_add, LineSize, barcolor[0]);
- drawColorLine(upc_img, 21 + upc_size_add, 4, 21 + upc_size_add, LineSize, barcolor[2]);
- drawColorLine(upc_img, 22 + upc_size_add, 4, 22 + upc_size_add, LineSize, barcolor[0]);
- drawColorLine(upc_img, 23 + upc_size_add, 4, 23 + upc_size_add, LineSize, barcolor[0]);
- drawColorLine(upc_img, 24 + upc_size_add, 4, 24 + upc_size_add, LineSize, barcolor[0]);
- drawColorLine(upc_img, 25 + upc_size_add, 4, 25 + upc_size_add, LineSize, barcolor[0]);
- drawColorLine(upc_img, 26 + upc_size_add, 4, 26 + upc_size_add, LineSize, barcolor[2]);
- drawColorLine(upc_img, 27 + upc_size_add, 4, 27 + upc_size_add, LineSize, barcolor[0]);
- drawColorLine(upc_img, 28 + upc_size_add, 4, 28 + upc_size_add, LineSize, barcolor[2]);
- drawColorLine(upc_img, 29 + upc_size_add, 4, 29 + upc_size_add, LineSize, barcolor[2]);
- drawColorLine(upc_img, 30 + upc_size_add, 4, 30 + upc_size_add, LineSize, barcolor[2]);
- drawColorLine(upc_img, 31 + upc_size_add, 4, 31 + upc_size_add, LineSize, barcolor[2]);
- drawColorLine(upc_img, 32 + upc_size_add, 4, 32 + upc_size_add, LineSize, barcolor[2]);
- drawColorLine(upc_img, 33 + upc_size_add, 4, 33 + upc_size_add, LineSize, barcolor[2]);
- drawColorLine(upc_img, 34 + upc_size_add, 4, 34 + upc_size_add, LineSize, barcolor[2]);
- drawColorLine(upc_img, 35 + upc_size_add, 4, 35 + upc_size_add, LineSize, barcolor[2]);
- drawColorLine(upc_img, 36 + upc_size_add, 4, 36 + upc_size_add, LineSize, barcolor[2]);
- new_upc_img = upc_preimg.resize(((37 + upc_size_add) * int(resize), (barheight[1] + 9) * int(resize)), Image.NEAREST); # use nearest neighbour
- del(upc_img);
- del(upc_preimg);
- upc_img = ImageDraw.Draw(new_upc_img);
+ end_barcolor = [1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0];
+ end_bc_num = 0;
+ end_b
+ while(end_bc_num < end_bc_num_end):
+  if(end_barcolor[end_bc_num]==1):
+   drawColorLine(upc_img, LineStart, 4, LineStart, LineSize, barwidth, barcolor[0], imageoutlib);
+  if(end_barcolor[end_bc_num]==0):
+   drawColorLine(upc_img, LineStart, 4, LineStart, LineSize, barwidth, barcolor[2], imageoutlib);
+  end_bc_num += 1;
+  LineStart += barwidth;
+  BarNum += 1;
+ if(pilsupport and imageoutlib=="pillow"):
+  new_upc_img = upc_preimg.resize((((37 * barwidth) + upc_size_add) * int(resize), (barheight[1] + 9) * int(resize)), Image.NEAREST); # use nearest neighbour
+  del(upc_img);
+  del(upc_preimg);
+  upc_img = ImageDraw.Draw(new_upc_img);
+ if(cairosupport and imageoutlib=="cairo"):
+  upc_imgpat = cairo.SurfacePattern(upc_preimg);
+  scaler = cairo.Matrix();
+  scaler.scale(1/int(resize),1/int(resize));
+  upc_imgpat.set_matrix(scaler);
+  upc_imgpat.set_filter(cairo.FILTER_NEAREST);
+  new_upc_preimg = cairo.ImageSurface(cairo.FORMAT_RGB24, ((37 * barwidth) + upc_size_add) * int(resize), (barheight[1] + 9) * int(resize));
+  new_upc_img = cairo.Context(new_upc_preimg);
+  new_upc_img.set_source(upc_imgpat);
+  new_upc_img.paint();
  if(not hidetext):
   NumTxtZero = 0; 
   LineTxtStart = 18;
   while (NumTxtZero < len(upc_print)):
-   drawColorText(upc_img, 10 * int(resize), LineTxtStart + (19 * (int(resize) - 1)), (barheight[0] + (barheight[0] * (int(resize) - 1)) + pil_addon_fix) + (textxy[1] * int(resize)), upc_print[NumTxtZero], barcolor[1]);
+   drawColorText(upc_img, 10 * int(resize), (LineTxtStart + (19 * (int(resize) - 1))) * barwidth, (barheight[0] + (barheight[0] * (int(resize) - 1)) + pil_addon_fix) + (textxy[1] * int(resize)), upc_print[NumTxtZero], barcolor[1], "ocrb", imageoutlib);
    LineTxtStart += 9 * int(resize);
    NumTxtZero += 1;
  del(upc_img);
- oldoutfile = upcean.getsfname.get_save_filename(outfile);
+ oldoutfile = upcean.barcodes.getsfname.get_save_filename(outfile);
  if(isinstance(oldoutfile, tuple) or isinstance(oldoutfile, list)):
   del(outfile);
   outfile = oldoutfile[0];
@@ -302,30 +310,40 @@ def create_code93_barcode(upc,outfile="./code93.png",resize=1,hideinfo=(False, F
  if(sys.version[0]=="2"):
   if(outfile=="-" or outfile=="" or outfile==" " or outfile is None):
    try:
-    if(pilsupport):
-     new_upc_img.save(sys.stdout, outfileext);
-    if(cairosupport):
+    if(pilsupport and imageoutlib=="pillow"):
+     if(outfileext=="BYTES"):
+      os.write(sys.stdout.fileno(), new_upc_img.tobytes()());
+     else:
+      new_upc_img.save(sys.stdout, outfileext);
+    if(cairosupport and imageoutlib=="cairo"):
      new_upc_preimg.write_to_png(sys.stdout);
    except:
     return False;
  if(sys.version[0]>="3"):
   if(outfile=="-" or outfile=="" or outfile==" " or outfile is None):
    try:
-    if(pilsupport):
-     new_upc_img.save(sys.stdout.buffer, outfileext);
-    if(cairosupport):
+    if(pilsupport and imageoutlib=="pillow"):
+     if(outfileext=="BYTES"):
+      os.write(sys.stdout.buffer.fileno(), new_upc_img.tobytes()());
+     else:
+      new_upc_img.save(sys.stdout.buffer, outfileext);
+    if(cairosupport and imageoutlib=="cairo"):
      new_upc_preimg.write_to_png(sys.stdout.buffer);
    except:
     return False;
  if(outfile!="-" and outfile!="" and outfile!=" "):
   try:
-   if(pilsupport):
-    new_upc_img.save(outfile, outfileext);
-   if(cairosupport):
+   if(pilsupport and imageoutlib=="pillow"):
+    if(outfileext=="BYTES"):
+     with open(outfile, 'wb+') as f:
+      f.write(new_upc_img.tobytes());
+    else:
+     new_upc_img.save(outfile, outfileext);
+   if(cairosupport and imageoutlib=="cairo"):
     new_upc_preimg.write_to_png(outfile);
   except:
    return False;
  return True;
 
-def draw_code93_barcode(upc,resize=1,hideinfo=(False, False, False),barheight=(48, 54),textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255))):
- return create_code93_barcode(upc,None,resize,hideinfo,barheight,textxy,barcolor);
+def draw_code93_barcode(upc,resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=1,textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
+ return create_code93_barcode(upc,None,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);
