@@ -11,28 +11,35 @@
     Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
     Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski
 
-    FileInfo: ean5.py - Last Update: 2/19/2023 Ver. 2.7.22 RC 1 - Author: cooldude2k 
+    FileInfo: ean5.py - Last Update: 3/8/2023 Ver. 2.7.24 RC 1 - Author: cooldude2k 
 '''
 
 from __future__ import absolute_import, division, print_function, unicode_literals;
-import re, os, sys, types, upcean.getsfname, upcean.support;
+import re, os, sys, types, upcean.barcodes.getsfname, upcean.support;
 pilsupport = upcean.support.check_for_pil();
+cairosupport = upcean.support.check_for_cairo();
+from upcean.barcodes.predraw import *;
 if(pilsupport):
- cairosupport = False;
-else:
- cairosupport = upcean.support.check_for_cairo();
-if(pilsupport):
- from upcean.barcodes.prepil import *;
- from PIL import Image, ImageDraw, ImageFont;
+ import upcean.barcodes.prepil;
 if(cairosupport):
- from upcean.precairo import *;
- import cairo;
+ import upcean.barcodes.precairo;
 
-def create_ean5_barcode_supplement(upc,outfile="./ean5_supplement.png",resize=1,hideinfo=(False, False, False),barheight=(48, 54),textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255))):
+def create_ean5_barcode_supplement(upc,outfile="./ean5_supplement.png",resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=1,textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
  upc = str(upc);
  hidesn = hideinfo[0];
  hidecd = hideinfo[1];
  hidetext = hideinfo[2];
+ imageoutlib = imageoutlib.lower();
+ if(not pilsupport and imageoutlib=="pillow"):
+  imageoutlib = "cairo";
+ if(not cairosupport and imageoutlib=="cairo"):
+  imageoutlib = "pillow";
+ if(imageoutlib!="pillow" and imageoutlib!="cairo"):
+  imageoutlib = "pillow";
+ if(not pilsupport and not cairosupport):
+  return False;
+ if(barwidth < 1): 
+  barwidth = 1;
  if(len(upc)>5 or len(upc)<5): 
   return False;
  upc_matches = re.findall("(\d{5})", upc);
@@ -40,48 +47,53 @@ def create_ean5_barcode_supplement(upc,outfile="./ean5_supplement.png",resize=1,
   return False;
  if(not re.findall("^([0-9]*[\.]?[0-9])", str(resize)) or int(resize) < 1):
   resize = 1;
- try:
-  pil_ver = Image.PILLOW_VERSION;
-  pil_ver = pil_ver.split(".");
-  pil_ver = [int(x) for x in pil_ver];
-  pil_is_pillow = True;
- except AttributeError:
+ if(pilsupport and imageoutlib=="pillow"):
   try:
-   pil_ver = Image.VERSION;
-   pil_is_pillow = False;
+   pil_ver = Image.PILLOW_VERSION;
+   pil_ver = pil_ver.split(".");
+   pil_ver = [int(x) for x in pil_ver];
+   pil_is_pillow = True;
   except AttributeError:
-   pil_ver = Image.__version__;
-   pil_is_pillow = True;
+   try:
+    pil_ver = Image.VERSION;
+    pil_is_pillow = False;
+   except AttributeError:
+    pil_ver = Image.__version__;
+    pil_is_pillow = True;
+   except NameError:
+    pil_ver = Image.__version__;
+    pil_is_pillow = True;
+   pil_ver = pil_ver.split(".");
+   pil_ver = [int(x) for x in pil_ver];
   except NameError:
-   pil_ver = Image.__version__;
-   pil_is_pillow = True;
-  pil_ver = pil_ver.split(".");
-  pil_ver = [int(x) for x in pil_ver];
- except NameError:
-  try:
-   pil_ver = Image.VERSION;
-   pil_is_pillow = False;
-  except AttributeError:
-   pil_ver = Image.__version__;
-   pil_is_pillow = True;
-  except NameError:
-   pil_ver = Image.__version__;
-   pil_is_pillow = True;
-  pil_ver = pil_ver.split(".");
-  pil_ver = [int(x) for x in pil_ver];
- pil_addon_fix = 0;
- pil_prevercheck = [str(x) for x in pil_ver];
- pil_vercheck = int(pil_prevercheck[0]+pil_prevercheck[1]+pil_prevercheck[2]);
- if(pil_is_pillow and pil_vercheck>=210 and pil_vercheck<220):
-  pil_addon_fix = int(resize) * 2;
+   try:
+    pil_ver = Image.VERSION;
+    pil_is_pillow = False;
+   except AttributeError:
+    pil_ver = Image.__version__;
+    pil_is_pillow = True;
+   except NameError:
+    pil_ver = Image.__version__;
+    pil_is_pillow = True;
+   pil_ver = pil_ver.split(".");
+   pil_ver = [int(x) for x in pil_ver];
+  pil_addon_fix = 0;
+  pil_prevercheck = [str(x) for x in pil_ver];
+  pil_vercheck = int(pil_prevercheck[0]+pil_prevercheck[1]+pil_prevercheck[2]);
+  if(pil_is_pillow and pil_vercheck>=210 and pil_vercheck<220):
+   pil_addon_fix = int(resize) * 2;
+ elif(pilsupport and imageoutlib=="pillow"):
+  pil_addon_fix = 0;
+ else:
+  pil_addon_fix = 0;
  LeftDigit = list(upc_matches[0]);
  CheckSum = (int(LeftDigit[0]) * 3) + (int(LeftDigit[1]) * 9) + (int(LeftDigit[2]) * 3) + (int(LeftDigit[3]) * 9) + (int(LeftDigit[4]) * 3);
  CheckSum = CheckSum % 10;
- if(pilsupport):
-  upc_preimg = Image.new("RGB", (56, barheight[1] + 9));
+ if(pilsupport and imageoutlib=="pillow"):
+  upc_preimg = Image.new("RGB", ((56 * barwidth), barheight[1] + 9));
   upc_img = ImageDraw.Draw(upc_preimg);
-  upc_img.rectangle([(0, 0), (56, barheight[1] + 9)], fill=barcolor[2]);
- if(cairosupport):
+  upc_img.rectangle([(0, 0), ((56 * barwidth), barheight[1] + 9)], fill=barcolor[2]);
+ if(cairosupport and imageoutlib=="cairo"):
   upc_preimg = cairo.ImageSurface(cairo.FORMAT_RGB24, 83, barheight[1] + 8);
   upc_img = cairo.Context (upc_preimg);
   upc_img.set_antialias(cairo.ANTIALIAS_NONE);
@@ -93,17 +105,19 @@ def create_ean5_barcode_supplement(upc,outfile="./ean5_supplement.png",resize=1,
  if(hidetext):
   LineSize = barheight[1];
  upc_array['code'].append( [0, 1, 0, 1, 1] );
- drawColorLine(upc_img, 0, 10, 0, LineSize, barcolor[2]);
- drawColorLine(upc_img, 1, 10, 1, LineSize, barcolor[0]);
- drawColorLine(upc_img, 2, 10, 2, LineSize, barcolor[2]);
- drawColorLine(upc_img, 3, 10, 3, LineSize, barcolor[0]);
- drawColorLine(upc_img, 4, 10, 4, LineSize, barcolor[0]);
+ start_barcolor = [0, 1, 0, 1, 1];
+ LineStart = 0;
+ BarNum = 0;
+ start_bc_num_end = len(start_barcolor);
+ while(BarNum < start_bc_num_end):
+  if(start_barcolor[BarNum]==1):
+   drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barwidth, barcolor[0], imageoutlib);
+  if(start_barcolor[BarNum]==0):
+   drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barwidth, barcolor[2], imageoutlib);
+  LineStart += barwidth;
+  BarNum += 1;
  NumZero = 0; 
- LineStart = 5;
  while (NumZero < len(LeftDigit)):
-  LineSize = barheight[0];
-  if(hidetext):
-   LineSize = barheight[1];
   left_barcolor_l = [0, 0, 0, 0, 0, 0, 0]; 
   left_barcolor_g = [1, 1, 1, 1, 1, 1, 1];
   if(int(LeftDigit[NumZero])==0): 
@@ -241,29 +255,43 @@ def create_ean5_barcode_supplement(upc,outfile="./ean5_supplement.png",resize=1,
   InnerUPCNum = 0;
   while (InnerUPCNum < len(left_barcolor)):
    if(left_barcolor[InnerUPCNum]==1):
-    drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barcolor[0]);
+    drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barwidth, barcolor[0], imageoutlib);
    if(left_barcolor[InnerUPCNum]==0):
-    drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barcolor[2]);
-   LineStart += 1;
+    drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barwidth, barcolor[2], imageoutlib);
+   LineStart += barwidth;
+   BarNum += 1;
    InnerUPCNum += 1;
   if(NumZero < 4):
-   drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barcolor[2]);
-   LineStart += 1;
-   drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barcolor[0]);
-   LineStart += 1;
+   drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barwidth, barcolor[2], imageoutlib);
+   LineStart += barwidth;
+   BarNum += 1;
+   drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barwidth, barcolor[0], imageoutlib);
+   LineStart += barwidth;
+   BarNum += 1;
   NumZero += 1;
- new_upc_img = upc_preimg.resize((56 * int(resize), (barheight[1] + 9) * int(resize)), Image.NEAREST);
- del(upc_img);
- del(upc_preimg);
- upc_img = ImageDraw.Draw(new_upc_img);
+ if(pilsupport and imageoutlib=="pillow"):
+  new_upc_img = upc_preimg.resize(((56 * barwidth) * int(resize), (barheight[1] + 9) * int(resize)), Image.NEAREST);
+  del(upc_img);
+  del(upc_preimg);
+  upc_img = ImageDraw.Draw(new_upc_img);
+ if(cairosupport and imageoutlib=="cairo"):
+  upc_imgpat = cairo.SurfacePattern(upc_preimg);
+  scaler = cairo.Matrix();
+  scaler.scale(1/int(resize),1/int(resize));
+  upc_imgpat.set_matrix(scaler);
+  upc_imgpat.set_filter(cairo.FILTER_NEAREST);
+  new_upc_preimg = cairo.ImageSurface(cairo.FORMAT_RGB24, (56 * barwidth) * int(resize), (barheight[1] + 9) * int(resize));
+  new_upc_img = cairo.Context(new_upc_preimg);
+  new_upc_img.set_source(upc_imgpat);
+  new_upc_img.paint();
  if(not hidetext):
-  drawColorText(upc_img, 10 * int(resize), 7 + (7 * (int(resize) - 1)), (barheight[0] + (barheight[0] * (int(resize) - 1)) + pil_addon_fix) + (textxy[1] * int(resize)), LeftDigit[0], barcolor[1]);
-  drawColorText(upc_img, 10 * int(resize), 16 + (15 * (int(resize) - 1)), (barheight[0] + (barheight[0] * (int(resize) - 1)) + pil_addon_fix) + (textxy[1] * int(resize)), LeftDigit[1], barcolor[1]);
-  drawColorText(upc_img, 10 * int(resize), 24 + (24 * (int(resize) - 1)), (barheight[0] + (barheight[0] * (int(resize) - 1)) + pil_addon_fix) + (textxy[1] * int(resize)), LeftDigit[2], barcolor[1]);
-  drawColorText(upc_img, 10 * int(resize), 32 + (32 * (int(resize) - 1)), (barheight[0] + (barheight[0] * (int(resize) - 1)) + pil_addon_fix) + (textxy[1] * int(resize)), LeftDigit[3], barcolor[1]);
-  drawColorText(upc_img, 10 * int(resize), 40 + (40 * (int(resize) - 1)), (barheight[0] + (barheight[0] * (int(resize) - 1)) + pil_addon_fix) + (textxy[1] * int(resize)), LeftDigit[4], barcolor[1]);
+  drawColorText(upc_img, 10 * int(resize), (7 + (7 * (int(resize) - 1))) * barwidth, (barheight[0] + (barheight[0] * (int(resize) - 1)) + pil_addon_fix) + (textxy[1] * int(resize)), LeftDigit[0], barcolor[1], "ocrb", imageoutlib);
+  drawColorText(upc_img, 10 * int(resize), (16 + (15 * (int(resize) - 1))) * barwidth, (barheight[0] + (barheight[0] * (int(resize) - 1)) + pil_addon_fix) + (textxy[1] * int(resize)), LeftDigit[1], barcolor[1], "ocrb", imageoutlib);
+  drawColorText(upc_img, 10 * int(resize), (24 + (24 * (int(resize) - 1))) * barwidth, (barheight[0] + (barheight[0] * (int(resize) - 1)) + pil_addon_fix) + (textxy[1] * int(resize)), LeftDigit[2], barcolor[1], "ocrb", imageoutlib);
+  drawColorText(upc_img, 10 * int(resize), (32 + (32 * (int(resize) - 1))) * barwidth, (barheight[0] + (barheight[0] * (int(resize) - 1)) + pil_addon_fix) + (textxy[1] * int(resize)), LeftDigit[3], barcolor[1], "ocrb", imageoutlib);
+  drawColorText(upc_img, 10 * int(resize), (40 + (40 * (int(resize) - 1))) * barwidth, (barheight[0] + (barheight[0] * (int(resize) - 1)) + pil_addon_fix) + (textxy[1] * int(resize)), LeftDigit[4], barcolor[1], "ocrb", imageoutlib);
  del(upc_img);
- oldoutfile = upcean.getsfname.get_save_filename(outfile);
+ oldoutfile = upcean.barcodes.getsfname.get_save_filename(outfile);
  if(isinstance(oldoutfile, tuple) or isinstance(oldoutfile, list)):
   del(outfile);
   outfile = oldoutfile[0];
@@ -289,22 +317,22 @@ def create_ean5_barcode_supplement(upc,outfile="./ean5_supplement.png",resize=1,
    return False;
  return True;
 
-def draw_ean5_barcode_supplement(upc,resize=1,hideinfo=(False, False, False),barheight=(48, 54),textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255))):
- return create_ean5_barcode_supplement(upc,None,resize,hideinfo,barheight,textxy,barcolor);
+def draw_ean5_barcode_supplement(upc,resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=1,textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
+ return create_ean5_barcode_supplement(upc,None,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);
 
-def create_ean5_barcode(upc,outfile="./ean5.png",resize=1,hideinfo=(False, False, False),barheight=(48, 54),textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255))):
+def create_ean5_barcode(upc,outfile="./ean5.png",resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=1,textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
  if(not re.findall("^([0-9]*[\.]?[0-9])", str(resize)) or int(resize) < 1):
   resize = 1;
- upc_preimg = Image.new("RGB", ((56 * int(resize)) + (8 * int(resize)), (barheight[1] + 9) * int(resize)));
+ upc_preimg = Image.new("RGB", (((56 * barwidth) * int(resize)) + (8 * int(resize)), (barheight[1] + 9) * int(resize)));
  upc_img = ImageDraw.Draw(upc_preimg);
- upc_img.rectangle([(0, 0), ((56 * int(resize)) + (8 * int(resize)), (barheight[1] + 9) * int(resize))], fill=barcolor[2]);
- upc_sup_img = create_ean5_barcode_supplement(upc,None,resize,hideinfo,barheight,textxy,barcolor);
+ upc_img.rectangle([(0, 0), (((56 * barwidth) * int(resize)) + (8 * int(resize)), (barheight[1] + 9) * int(resize))], fill=barcolor[2]);
+ upc_sup_img = create_ean5_barcode_supplement(upc,None,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);
  if(upc_sup_img is None or isinstance(upc_sup_img, bool)):
   return False;
  upc_preimg.paste(upc_sup_img,(8 * int(resize),0));
  del(upc_sup_img);
  del(upc_img);
- oldoutfile = upcean.getsfname.get_save_filename(outfile);
+ oldoutfile = upcean.barcodes.getsfname.get_save_filename(outfile);
  if(isinstance(oldoutfile, tuple) or isinstance(oldoutfile, list)):
   del(outfile);
   outfile = oldoutfile[0];
@@ -314,30 +342,40 @@ def create_ean5_barcode(upc,outfile="./ean5.png",resize=1,hideinfo=(False, False
  if(sys.version[0]=="2"):
   if(outfile=="-" or outfile=="" or outfile==" " or outfile is None):
    try:
-    if(pilsupport):
-     new_upc_img.save(sys.stdout, outfileext);
-    if(cairosupport):
+    if(pilsupport and imageoutlib=="pillow"):
+     if(outfileext=="BYTES"):
+      os.write(sys.stdout.fileno(), new_upc_img.tobytes()());
+     else:
+      new_upc_img.save(sys.stdout, outfileext);
+    if(cairosupport and imageoutlib=="cairo"):
      new_upc_preimg.write_to_png(sys.stdout);
    except:
     return False;
  if(sys.version[0]>="3"):
   if(outfile=="-" or outfile=="" or outfile==" " or outfile is None):
    try:
-    if(pilsupport):
-     new_upc_img.save(sys.stdout.buffer, outfileext);
-    if(cairosupport):
+    if(pilsupport and imageoutlib=="pillow"):
+     if(outfileext=="BYTES"):
+      os.write(sys.stdout.buffer.fileno(), new_upc_img.tobytes()());
+     else:
+      new_upc_img.save(sys.stdout.buffer, outfileext);
+    if(cairosupport and imageoutlib=="cairo"):
      new_upc_preimg.write_to_png(sys.stdout.buffer);
    except:
     return False;
  if(outfile!="-" and outfile!="" and outfile!=" "):
   try:
-   if(pilsupport):
-    new_upc_img.save(outfile, outfileext);
-   if(cairosupport):
+   if(pilsupport and imageoutlib=="pillow"):
+    if(outfileext=="BYTES"):
+     with open(outfile, 'wb+') as f:
+      f.write(new_upc_img.tobytes());
+    else:
+     new_upc_img.save(outfile, outfileext);
+   if(cairosupport and imageoutlib=="cairo"):
     new_upc_preimg.write_to_png(outfile);
   except:
    return False;
  return True;
 
-def draw_ean5_barcode(upc,resize=1,hideinfo=(False, False, False),barheight=(48, 54),textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255))):
- return create_ean5_barcode(upc,None,resize,hideinfo,barheight,textxy,barcolor);
+def draw_ean5_barcode(upc,resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=1,textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
+ return create_ean5_barcode(upc,None,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);
