@@ -11,7 +11,7 @@
     Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
     Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski
 
-    $FileInfo: files.py - Last Update: 2/19/2023 Ver. 2.7.22 RC 1 - Author: cooldude2k $
+    $FileInfo: files.py - Last Update: 3/8/2023 Ver. 2.7.24 RC 1 - Author: cooldude2k $
 '''
 
 from __future__ import absolute_import, division, print_function, unicode_literals;
@@ -96,6 +96,8 @@ def create_barcode_from_xml_file(xmlfile, draw=False):
     xmlbarcode.update({"hideinfo": tuple(hidebcinfoval)});
    if('height' in child.attrib):
     xmlbarcode.update({"barheight": tuple(map(int, child.attrib['height'].split()))});
+   if('width' in child.attrib):
+    xmlbarcode.update({"barwidth": tuple(map(int, child.attrib['width'].split()))});
    if('textxy' in child.attrib):
     xmlbarcode.update({"textxy": tuple(map(int, child.attrib['textxy'].split()))});
    if('color' in child.attrib):
@@ -123,6 +125,8 @@ def create_barcode_from_xml_file(xmlfile, draw=False):
     colorlist3 = (int(colorsplit3[0], 16), int(colorsplit3[1], 16), int(colorsplit3[2], 16));
     colorlist = (colorlist1, colorlist2, colorlist3);
     xmlbarcode.update({"barcolor": colorlist});
+   if('imageoutlib' in child.attrib):
+    xmlbarcode.update({"imageoutlib": tuple(map(int, child.attrib['imgoutlib'].split()))});
    bcstatinfo = upcean.barcodes.shortcuts.validate_create_barcode(**xmlbarcode);
    if(draw or 'file' not in child.attrib):
     bcdrawlist.append(bcstatinfo);
@@ -170,10 +174,14 @@ def convert_from_xml_to_json_file(xmlfile, jsonfile=None):
    jsontmpdict.update({"hideinfo": child.attrib['hideinfo']});
   if('height' in child.attrib):
    jsontmpdict.update({"height": child.attrib['height']});
+  if('width' in child.attrib):
+   jsontmpdict.update({"width": child.attrib['width']});
   if('textxy' in child.attrib):
    jsontmpdict.update({"textxy": child.attrib['textxy']});
   if('color' in child.attrib):
    jsontmpdict.update({"color": child.attrib['color']});
+  if('imageoutlib' in child.attrib):
+   jsontmpdict.update({"imageoutlib": child.attrib['imageoutlib']});
   jsonlist['barcodes']['barcode'].append(jsontmpdict);
  if(jsonfile is not None):
   jsonofile = open(jsonfile, "w+");
@@ -229,6 +237,8 @@ def create_barcode_from_json_file(jsonfile, draw=False):
    jsonbarcode.update({"hideinfo": tuple(hidebcinfoval)});
   if('height' in bctree[bctreect]):
    jsonbarcode.update({"barheight": tuple(map(int, bctree[bctreect]['height'].split()))});
+  if('width' in bctree[bctreect]):
+   jsonbarcode.update({"barwidth": tuple(map(int, bctree[bctreect]['width'].split()))});
   if('textxy' in bctree[bctreect]):
    jsonbarcode.update({"textxy": tuple(map(int, bctree[bctreect]['textxy'].split()))});
   if('color' in bctree[bctreect]):
@@ -256,6 +266,8 @@ def create_barcode_from_json_file(jsonfile, draw=False):
    colorlist3 = (int(colorsplit3[0], 16), int(colorsplit3[1], 16), int(colorsplit3[2], 16));
    colorlist = (colorlist1, colorlist2, colorlist3);
    jsonbarcode.update({"barcolor": colorlist});
+  if('imageoutlib' in bctree[bctreect]):
+   jsonbarcode.update({"imageoutlib": tuple(map(int, bctree[bctreect]['imgoutlib'].split()))});
   bcstatinfo = upcean.barcodes.shortcuts.validate_create_barcode(**jsonbarcode);
   if(draw or 'file' not in bctree[bctreect]):
    bcdrawlist.append(bcstatinfo);
@@ -378,6 +390,12 @@ def create_barcode_from_qs_file(qsfile, draw=False):
    pass;
   except IndexError:
    pass;
+  try:
+   qsbarcode.update({"barwidth": int(bctree['width'][bctreect])});
+  except KeyError:
+   pass;
+  except IndexError:
+   pass;
   try:
    qsbarcode.update({"textxy": int(bctree['textxy'][bctreect])});
   except KeyError:
@@ -413,6 +431,12 @@ def create_barcode_from_qs_file(qsfile, draw=False):
    pass;
   except IndexError:
    pass;
+  try:
+   qsbarcode.update({"imageoutlib": int(bctree['imgoutlib'][bctreect])});
+  except KeyError:
+   pass;
+  except IndexError:
+   pass;
   bcstatinfo = upcean.barcodes.shortcuts.validate_create_barcode(**qsbarcode);
   if(draw or nofilesave ):
    bcdrawlist.append(bcstatinfo);
@@ -480,6 +504,12 @@ def convert_from_qs_to_xml_file(qsfile, xmlfile=None):
    pass;
   except IndexError:
    pass;
+  try:
+   qsbarcode.update({"barwidth": bctree['barwidth'][bctreect]});
+  except KeyError:
+   pass;
+  except IndexError:
+   pass;
   try:
    qsbarcode.update({"textxy": bctree['textxy'][bctreect]});
   except KeyError:
@@ -492,6 +522,12 @@ def convert_from_qs_to_xml_file(qsfile, xmlfile=None):
    pass;
   except IndexError:
    pass;
+  try:
+   qsbarcode.update({"imageoutlib": bctree['imageoutlib'][bctreect]});
+  except KeyError:
+   pass;
+  except IndexError:
+   pass;
   upcxml.characters(" ");
   upcxml.startElement("barcode", qsbarcode);
   upcxml.endElement("barcode");
@@ -556,6 +592,12 @@ def convert_from_qs_to_json_file(qsfile, jsonfile=None):
    pass;
   except IndexError:
    pass;
+  try:
+   qsbarcode.update({"barwidth": bctree['barwidth'][bctreect]});
+  except KeyError:
+   pass;
+  except IndexError:
+   pass;
   try:
    qsbarcode.update({"textxy": bctree['textxy'][bctreect]});
   except KeyError:
@@ -568,6 +610,12 @@ def convert_from_qs_to_json_file(qsfile, jsonfile=None):
    pass;
   except IndexError:
    pass;
+  try:
+   qsbarcode.update({"imageoutlib": bctree['imageoutlib'][bctreect]});
+  except KeyError:
+   pass;
+  except IndexError:
+   pass;
   jsonlist['barcodes']['barcode'].append(qsbarcode);
   bctreect = bctreect + 1;
  if(jsonfile is not None):
