@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*- 
-
 # -*- coding: utf-8 -*-
 '''
     This program is free software; you can redistribute it and/or modify
@@ -13,7 +11,7 @@
     Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
     Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski
 
-    $FileInfo: getprefix.py - Last Update: 2/19/2023 Ver. 2.7.22 RC 1 - Author: cooldude2k $
+    $FileInfo: getprefix.py - Last Update: 3/8/2023 Ver. 2.7.24 RC 1 - Author: cooldude2k $
 '''
 
 from __future__ import absolute_import, division, print_function, unicode_literals;
@@ -392,6 +390,8 @@ def get_goodwill_upca_barcode_info(upc, infotype=None):
   gw_item_type = "Furniture";
  elif(re.findall("^(4120120)", upc)):
   gw_item_type = "Books";
+ elif(re.findall("^(4002000)", upc)):
+  gw_item_type = "Mystery Dozen Deal";
  else:
   gw_item_type = None;
  gw_item_color = None;
@@ -409,6 +409,12 @@ def get_goodwill_upca_barcode_info(upc, infotype=None):
   gw_item_color = None;
  price_matches = re.findall("^(\d{3})(\d{2})", upc_matches[1]);
  price_matches = price_matches[0];
+ if(price_matches[0]=="699" and gw_item_type=="Mystery Dozen Deal"):
+  gw_item_type = "Mystery 1/2 Dozen Deal";
+ elif(price_matches[0]=="999" and gw_item_type=="Mystery Dozen Deal"):
+  gw_item_type = "Mystery Dozen Deal";
+ else:
+  gw_item_type = gw_item_type;
  price_alt = str(price_matches[0].lstrip('0'))+price_matches[1];
  formated_price = price_matches[0]+"."+price_matches[1];
  formated_price_alt = str(price_matches[0].lstrip('0'))+"."+price_matches[1];
@@ -455,7 +461,7 @@ def get_goodwill_upca_barcode_checkdigit(upc):
  return product.get("checkdigit", False);
 
 '''
-// Get variable weight info
+// Get variable weight info from UPC-A
 // Source: http://wiki.answers.com/Q/How_does_a_price_embedded_bar_code_work
 // Source: http://en.wikipedia.org/wiki/Universal_Product_Code#Prefixes
 '''
@@ -475,36 +481,105 @@ def get_upca_vw_barcode_info(upc, infotype=None):
   return product;
  if(infotype is not None):
   return product.get(infotype, product);
+def get_vw_barcode_info(upc, infotype=None):
+ return get_upca_vw_barcode_info(upc, infotype);
 def get_upca_vw_barcode_numbersystem(upc):
  upc = str(upc);
  product = get_upca_vw_barcode_info(upc);
  if(not product):
   return False;
  return product.get("numbersystem", False);
+def get_vw_barcode_numbersystem(upc):
+ return get_upca_vw_barcode_numbersystem(upc);
 def get_upca_vw_barcode_code(upc):
  upc = str(upc);
  product = get_upca_vw_barcode_info(upc);
  if(not product):
   return False;
  return product.get("code", False);
+def get_vw_barcode_code(upc):
+ return get_upca_vw_barcode_code(upc);
 def get_upca_vw_barcode_price(upc):
  upc = str(upc);
  product = get_upca_vw_barcode_info(upc);
  if(not product):
   return False;
  return product.get("price", False);
+def get_vw_barcode_price(upc):
+ return get_upca_vw_barcode_price(upc);
 def get_upca_vw_barcode_pricecs(upc):
  upc = str(upc);
  product = get_upca_vw_barcode_info(upc);
  if(not product):
   return False;
  return product.get("pricecs", False);
+def get_vw_barcode_pricecs(upc):
+ return get_upca_vw_barcode_pricecs(upc);
 def get_upca_vw_barcode_checkdigit(upc):
  upc = str(upc);
  product = get_upca_vw_barcode_info(upc);
  if(not product):
   return False;
  return product.get("checkdigit", False);
+def get_upca_vw_barcode_checksum(upc):
+ return get_upca_vw_barcode_checkdigit(upc);
+def get_vw_barcode_checkdigit(upc):
+ return get_upca_vw_barcode_checkdigit(upc);
+def get_vw_barcode_checksum(upc):
+ return get_upca_vw_barcode_checkdigit(upc);
+
+'''
+// Get variable weight info from EAN-13
+// Source: https://softmatic.com/barcode-ean-13.html#ean-country
+'''
+def get_ean13_vw_barcode_info(upc, infotype=None):
+ upc = str(upc);
+ if(re.findall("^0(\d{13})", upc)):
+  upc_matches = re.findall("^0(\d{13})", upc);
+  upc = upc_matches[0];
+ if(not re.findall("^(\d{13})", upc)):
+  return False;
+ if(not re.findall("^2(\d{12})", upc)):
+  return False;
+ upc_matches = re.findall("^2(\d{1})(\d{5})(\d{5})(\d{1})", upc);
+ upc_matches = upc_matches[0];
+ product = {'numbersystem': str(2), 'subnumbersystem': upc_matches[0], 'code': upc_matches[1], 'price': upc_matches[2], 'checkdigit': upc_matches[3]};
+ if(infotype is None):
+  return product;
+ if(infotype is not None):
+  return product.get(infotype, product);
+def get_ean13_vw_barcode_numbersystem(upc):
+ upc = str(upc);
+ product = get_ean13_vw_barcode_info(upc);
+ if(not product):
+  return False;
+ return product.get("numbersystem", False);
+def get_ean13_vw_barcode_subnumbersystem(upc):
+ upc = str(upc);
+ product = get_ean13_vw_barcode_info(upc);
+ if(not product):
+  return False;
+ return product.get("subnumbersystem", False);
+def get_ean13_vw_barcode_code(upc):
+ upc = str(upc);
+ product = get_ean13_vw_barcode_info(upc);
+ if(not product):
+  return False;
+ return product.get("code", False);
+def get_ean13_vw_barcode_price(upc):
+ upc = str(upc);
+ product = get_ean13_vw_barcode_info(upc);
+ if(not product):
+  return False;
+ return product.get("price", False);
+def get_ean13_vw_barcode_checkdigit(upc):
+ upc = str(upc);
+ product = get_ean13_vw_barcode_info(upc);
+ if(not product):
+  return False;
+ return product.get("checkdigit", False);
+def get_ean13_vw_barcode_checksum(upc):
+ return get_ean13_vw_barcode_checkdigit(upc);
 
 '''
 // Get coupon info
